#ifndef LIBNMOS_H
#define LIBNMOS_H

#define NM_VPN_API_SUBJECT_TO_CHANGE	/* as required by nm-vpn-ui-interface.h */


#include <glib.h>
#include <gtk/gtk.h>
#include <gnome.h>
#include <glade/glade.h>

#include <NetworkManager/nm-vpn-ui-interface.h>


/* Lets Define The Message Strings that identify
 * each Arg passed to nm-os on the DBUS
 */

#define NMOS_STRING_NAME		             "Name"
#define NMOS_STRING_REMOTE_GATEWAY           "Remote Gateway"             /* Remote gateway */
#define NMOS_STRING_REMOTE_NETWORK		     "Remote Network"             /* Remote Network */
#define NMOS_STRING_REMOTE_SUBNET_MASK       "Subnet Mask"                /* Subnet Mask */
#define NMOS_STRING_AUTH_METHOD		         "Auth Method"                /* Auth Method */
#define NMOS_STRING_AUTH_SECRET              "Auth Secret"
#define NMOS_STRING_XAUTH		             "Xauth"
#define NMOS_STRING_XAUTH_USERNAME		     "Xauth Username"
#define NMOS_STRING_XAUTH_PASSWORD	 	     "Xauth Password"
#define NMOS_STRING_PFS		                 "PFS"
#define NMOS_STRING_DPD		                 "DPD"
#define NMOS_STRING_NATT		             "Nat-T"
#define NMOS_STRING_REPLAY_DETECTION		 "Replay Detection"
#define NMOS_STRING_AUTO_KEEPALIVE		     "Auto KeepAlive"
#define NMOS_STRING_KEEP_ALIVE_FREQUENCY	 "Keep Alive Frequency"
#define NMOS_STRING_INTERFACE_MODE		     "Interface Mode"
#define NMOS_STRING_INTERFACE_NAME		     "Interface: Name"
#define NMOS_STRING_INTERFACE_DHCP		     "Interface: DHCP"
#define NMOS_STRING_INTERFACE_IP_ADDRESS     "Interface: IP Address"
#define NMOS_STRING_INTERFACE_SUBNET_MASK	 "Interface: Subnet Mask"
#define NMOS_STRING_INTERFACE_DNS_SERVER     "Interface: DNS Server"
#define NMOS_STRING_INTERFACE_WINS_SERVER    "Interface: WINS Server"
#define NMOS_STRING_PHASE1_MODE		         "Phase1: Mode"
#define NMOS_STRING_PHASE1_DH_GROUP_1		 "Phase1: DH Group 1"
#define NMOS_STRING_PHASE1_DH_GROUP_2		 "Phase1: DH Group 2"
#define NMOS_STRING_PHASE1_DH_GROUP_5		 "Phase1: DH Group 5"
#define NMOS_STRING_PHASE1_KEYLIFE		     "Phase1: Keylife"
#define NMOS_STRING_PHASE1_LOCAL_ID		     "Phase1: LocalID"
#define NMOS_STRING_PHASE1_AUTH		         "Phase1: Auth"
#define NMOS_STRING_PHASE1_ENC		         "Phase1: Enc"
#define NMOS_STRING_PHASE2_DH_GROUP		     "Phase2: DH Group"
#define NMOS_STRING_PHASE2_KEYLIFE		     "Phase2: Keylife"
#define NMOS_STRING_PHASE2_AUTH		         "Phase2: Auth"
#define NMOS_STRING_PHASE2_ENC		         "Phase2: Enc"
#define NMOS_STRING_VAL_ERR                  "VAL_ERR"
#define NMOS_STRING_VAL_NONE                 "<NONE>"


//global helper variables.
guint newProp_source;
gboolean HAVE_DATA_LIST;

//define a struct to hold data obtained through the UI
typedef struct _UIdata {
	//Build Properties Widgets Data Containers
	GString          *name;
	GString          *gateway;
	GString          *remote_network;
	GString          *remote_netmask;
	gboolean         radiobutton_psk;
	gboolean         radiobutton_cert;
	GString          *PSK;
	//insert interface holders here
	gboolean         interfaceMode;
	GString          *interface_name;
	gboolean    	 vip_dhcp;
	gboolean    	 vip_manual;
	GString     	 *vip_IP;
	GString     	 *vip_SnM;
	GString     	 *vip_DNS;
	GString     	 *vip_WINS;
	//insert Xauth holders here
	gboolean    	 xauth;
	gboolean    	 promptXauth;
	GString     	 *xauthUsername;
	GString     	 *xauthPassword;
	//phase1 data holders here
	gboolean    	 mainmode;
	gboolean    	 aggressivemode;
	gboolean    	 p1_dh1;
	gboolean    	 p1_dh2;
	gboolean    	 p1_dh5;
	GtkListStore   	 *P1_ProposalList;
	GString     	 *P1_keylife;
	GString     	 *P1_localid;
	//phase2 data holders here
	gboolean    	 p2_dh1;
	gboolean    	 p2_dh2;
	gboolean    	 p2_dh5;
	GString     	 *P2_keylife;
	GtkListStore  	 *P2_ProposalList;
	//advanced options data holders here
	gboolean    	 PFS;
	gboolean    	 auto_keepalive;
	GString     	 *keepalivefreq;
	gboolean    	 NatT;
	gboolean    	 DPD;
	gboolean    	 ReplayDetection;
} UIdata;


//Define pointers to the Data widgets in the UI we're implementing.
typedef struct _NetworkManagerVpnUIImpl {
	NetworkManagerVpnUI parent;
	NetworkManagerVpnUIDialogValidityCallback callback;
	
	gpointer           callback_user_data;
	gboolean           HAVE_DATA_LIST;
	
	UIdata            *params;
	GladeXML          *xml;
	GtkWidget         *widget;
	GtkWidget         *proposal_widget; //popup
	GtkWidget         *xauth_widget; //popup
	GtkWidget         *interface_widget; //popup
	
	GtkButton         *w_Import;
	GtkEntry          *w_name;
	GtkEntry          *w_gateway;
	GtkEntry          *w_remote_network;
	GtkEntry          *w_remote_netmask;
	GtkRadioButton    *w_radiobutton_psk;
	GSList            *w_radiobutton_psk_group;
	GtkRadioButton    *w_radiobutton_cert;
	GtkEntry          *w_PSK;
	//insert interface holders here
	GtkCheckButton   *w_interfaceMode;
	GtkButton         *w_configInterface;
	GtkComboBox       *w_interface_name;
	GtkRadioButton    *w_vip_dhcp;
	GtkRadioButton    *w_vip_manual;
	GtkEntry          *w_vip_IP;
	GtkEntry          *w_vip_SnM;
	GtkEntry          *w_vip_DNS;
	GtkEntry          *w_vip_WINS;
	//insert Xauth holders here
	GtkCheckButton   *w_xauth;
	GtkButton         *w_configXauth;
	GtkCheckButton   *w_promptXauth;
	GtkEntry          *w_xauthUsername;
	GtkEntry          *w_xauthPassword;
	//phase1 data
	GtkButton         *w_P1_Add;
	GtkButton         *w_P1_Delete;
	GtkButton         *w_P1_DeleteAll;
	GtkRadioButton    *w_mainmode;
	GtkRadioButton    *w_aggressivemode;
	GtkCheckButton   *w_p1_dh1;
	GtkCheckButton   *w_p1_dh2;
	GtkCheckButton   *w_p1_dh5;
	GtkTreeView       *w_P1_ProposalList;
	GtkEntry          *w_P1_keylife;
	GtkEntry          *w_P1_localid;
	//phase2 data
	GtkButton         *w_P2_Add;
	GtkButton         *w_P2_Delete;
	GtkButton         *w_P2_DeleteAll;
	GtkRadioButton    *w_P2_dh1;
	GSList            *w_P2_dh1_group;
	GtkRadioButton    *w_P2_dh2;
	GtkRadioButton    *w_P2_dh5;
	GtkEntry          *w_P2_keylife;
	GtkTreeView       *w_P2_ProposalList;
	//advanced options
	GtkCheckButton   *w_PFS;
	GtkToggleButton   *w_auto_keepalive;
	GtkEntry          *w_keepalivefreq;
	GtkCheckButton   *w_NatT;
	GtkCheckButton   *w_DPD;
	GtkCheckButton   *w_ReplayDetection;
	
	//control buttons
	GtkButton         *w_newconn_OK;
	GtkButton         *w_newconn_Apply;
	GtkButton         *w_newconn_Cancel;
	
	//Xauth control buttons
	GtkButton         *w_confXauthOK;
	GtkButton         *w_confXauthCancel;
	
	//interface control buttons
	GtkExpander       *w_confInt_Expander;
	GtkButton         *w_confIntOK;
	GtkButton         *w_confIntCANCEL;

	//proposal control buttons
	GtkButton         *w_newProp_OK;
	GtkButton         *w_newProp_Reset;
	GtkButton         *w_newProp_Cancel;
	GtkTextView       *w_prop_Preview;
	GtkComboBox       *w_newprop_enc;
	GtkComboBox       *w_newprop_auth;
	
	GtkExpander         *w_p1_expander;
	GtkExpander         *w_p2_expander;
} NetworkManagerVpnUIImpl;


#endif
