#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <glib/gi18n-lib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gnome.h>
#include <glade/glade.h>

#include "libnm-os-xauth.h"

typedef struct _XauthUIdata {
	gchar *username;
	gchar *password;
} XauthUIdata;

//Define pointers to the Data widgets in the UI we're implementing.
typedef struct _NmOsXauthUI {
	gboolean			HaveCredentials;
	
	XauthUIdata			*params;
	GladeXML			*xml;
	GtkWidget			*widget;
	GtkButton			*w_xauth_ok;
	GtkButton			*w_xauth_cancel;
	GtkEntry			*w_username;
	GtkEntry			*w_password;
} XauthUI;



void
on_xauth_ok_clicked (GtkButton       *button, gpointer         user_data)
{
	XauthUI *impl = (XauthUI*) user_data;
	impl->params->username = gtk_entry_get_text (impl->w_username);
	impl->params->password = gtk_entry_get_text (impl->w_password);
	gtk_widget_destroy (impl->widget);
	impl->HaveCredentials = TRUE;
	gtk_main_quit();
	return;
}


void
on_xauth_cancel_clicked (GtkButton       *button, gpointer         user_data)
{
	XauthUI *impl = (XauthUI*) user_data;
	gtk_widget_destroy (impl->widget);
	impl->HaveCredentials = FALSE;
	gtk_main_quit();
	return;
}


//exported Funtion call to prompt for Xauth Credentials in X.
gboolean prompt_xauth (gchar *username, gchar *password)
{
	char *glade_file;
	XauthUI *impl;
	
	impl = g_new0 (XauthUI, 1);
	impl->params = g_new0 (XauthUIdata, 1);
	//unless the user clicks okay, we don't have the credentials
	impl->HaveCredentials = FALSE;
	
	glade_file = g_strdup_printf ("%s/%s", GLADEDIR, "libnm-os-xauth.la.glade");
	impl->xml = glade_xml_new (glade_file, NULL, GETTEXT_PACKAGE);
	
	g_free (glade_file);
	if (impl->xml == NULL)
		return impl->HaveCredentials;
		
	impl->widget = glade_xml_get_widget (impl->xml, "xauth_dialog");
	impl->w_username = GTK_ENTRY (glade_xml_get_widget (impl->xml, "xauth_uname"));
	impl->w_password = GTK_ENTRY (glade_xml_get_widget (impl->xml, "xauth_passwd"));
	impl->w_xauth_ok              = GTK_BUTTON (glade_xml_get_widget (impl->xml, "xauth_ok"));
	impl->w_xauth_cancel      = GTK_BUTTON (glade_xml_get_widget (impl->xml, "xauth_cancel"));

	g_signal_connect ((gpointer) impl->w_xauth_ok, "clicked",
						G_CALLBACK (on_xauth_ok_clicked),
						impl);

	g_signal_connect ((gpointer) impl->w_xauth_cancel, "clicked",
						G_CALLBACK (on_xauth_cancel_clicked),
						impl);

		/* make the widget reusable */
	gtk_signal_connect (GTK_OBJECT (impl->widget), "delete-event", 
			    GTK_SIGNAL_FUNC (gtk_widget_hide_on_delete), NULL);

	gtk_widget_show_all (impl->widget);
	gtk_main ();
	
	if (impl->HaveCredentials)
	{
		username = g_strdup_printf ("%s", impl->params->username);
		password = g_strdup_printf ("%s", impl->params->password);
	}
	else
	{
		username = NULL;
		password = NULL;
	}
	return impl->HaveCredentials;
}

int main (int argc, char *argv[] )
{
	g_type_init();

	if (argc !=3 )
	{
		printf ("error: arg count was %d, when it has to be 2!", (argc-1));
		exit (-1);
	}
	gchar *uname = argv[1];
	gchar *pword = argv[2];

	if (!prompt_xauth(uname, pword))
		return -1;
	else 
	{
		printf ("\n   Username: %s, Password: %s \n", uname, pword);
	}
	return 0;
}
